/* Configuration constants
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Bootloader

The Bootloader is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

The Bootloader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef CONFIG_H
#define CONFIG_H

#include <inttypes.h>

/** \file config.h
 * \brief Bootloader Configuration
 */

/* Communication */

/** \addtogroup communicationbuf
 * @{
 */

/** \brief header size */
#define COMM_PKT_HEADER_SIZE 4

/** \brief buffer size
 * \note match against host loader */
#define COMM_PKT_DATA_SIZE 8187

#define COMM_MAX_READ_SIZE (2 * COMM_PKT_DATA_SIZE)

#define COMM_PKT_SIZE (COMM_PKT_HEADER_SIZE + COMM_PKT_DATA_SIZE)

/** \brief location of the serial number of the device
 * \see communicationStart */
#define COMM_SERIAL_NUMBER_LOC (0x40000ul - 4)

/** \brief location of the node id of the device
 * \see communicationStart */
#define COMM_NODE_ID_LOC (0x40000ul - 8)

/** \brief min address which can be flashed
 * \see setAddress
 */
#define COMM_FLASH_PROG_MIN 0x40000ul



/* USB Configuration */

/** \brief Description String for USB Interface */
#define USB_COMM_INTERFACE_DESC "Flash Interface"

/** \brief Maximum Power Consumption of Device in \c mA */
#define USB_COMM_MAX_POWER 200

/** \brief Description String for USB Configuration */
#define USB_COMM_CONFIG_DESC "Flash Configuration"

/** \brief Vendor ID (Intel) */
#define USB_COMM_ID_VENDOR 0x042b

/** \brief Product ID (taken from old loader) */
#define USB_COMM_ID_PRODUCT 0x1337

/** \brief Device Version number in \c BCD */
#define USB_COMM_DEVICE_VERSION 0x0202

/** \brief Vendor Description String */
#define USB_COMM_VENDOR_DESC "Intel"

/** \brief Product Description String */
#define USB_COMM_PRODUCT_DESC "Imote 2"

/** \brief Language ID (united states) */
#define USB_COMM_LANGUAGE_ID 0x0409

/** @} */



/* Debug */

/** \addtogroup debug
 * @{
 */

/** \brief buffer size for debugf() message
 * \ingroup debug
 * \see debugf */
#define DEBUG_BUFFER_SIZE 200

/** @} */



/* Flash */

/** \addtogroup flash
 * @{
 */

/** \brief flash block size in second partition */
#define FLASH_BLOCK_SIZE 0x20000

/** \brief overall flash size (32M) */
#define FLASH_SIZE 0x2000000

/** \brief chip base address */
#define FLASH_BASE_ADDR 0x0

/** @} */



/* Main / Utils */

/** \brief program start
 * for more image related symbols, see file image.c */
#define IMG_DEFAULT_LOC 0x00400000ul

/** \brief wait for host loader for X seconds
 * \see utilStartBootTimeout */
#define BOOT_TIMEOUT 3

/** \brief number of seconds to hold down the button to reboot
 * \see pmicInterrupt */
#define REBOOT_TIMEOUT 1

/** \brief switch to idle mode between running tasks
 * \see main
 */
#define POWER_SAVE TRUE

/** \brief power up core frequency while being in UPLOADING state */
#define POWER_UP_UPLOADING FALSE

#endif

