/* Driver framework
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Bootloader

The Bootloader is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

The Bootloader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef DRIVER_H
#define DRIVER_H

#include <inttypes.h>

/** \file driver.h
 * \brief Generic Driver routines
 */

/** Structure for a driver or subsystem */
struct Driver {
	/** initialization routine. Set up registers or memory but do not start the
	 *  subsystem. Can be null iff there is no initialization */
	void (*init)(struct Driver*);
	/** start routine. Start the subsystem (e.g. enable interrupts).
		Can be null iff there is nothing to do */
	void (*start)(struct Driver*);
	/** stop routine. Stop the subsystem, such that it can be restarted by
	 * using the start routine. Can be null iff there is nothing to do */
	void (*stop)(struct Driver*);
	/** cleanup routine. Revert the effects of the initialization routine.
	 *  Can be null iff there is no cleanup */
	void (*finish)(struct Driver*);
	
	/** driver name. required */
	const char * name;
	
	/** true iff driver is initialized */
	bool isInit;
	/** true iff driver is started */
	bool isStart;
	
	/** user defined data */
	void * data;
};

void driverInit(struct Driver * driver);
void driverStart(struct Driver * driver);
void driverStop(struct Driver * driver);
void driverFinish(struct Driver * driver);

#endif

