
#include <inttypes.h>
#include "core/mmu.h"
#include "core/cache.h"
#include "debug.h"

__attribute__ ((aligned(0x4000)))
uint32_t mmuTable[4 * 1024];

static inline void mapTable(uint32_t * tbl, uint32_t start, uint32_t len,
	uint8_t domain, bool cacheable, bool bufferable) {
	uint32_t i;
		
	for (i = start; i < start + len; ++i) {
		tbl[i] = (i << 20)
			| ((domain & 0xf) << 5)
			| (cacheable ? (1 << 3) : 0)
			| (bufferable ? (1 << 2) : 0)
			| 0x2;
	}
}

static inline void voidTable(uint32_t * tbl, uint32_t start, uint32_t len) {
	uint32_t i;
		
	for (i = start; i < start + len; ++i) {
		tbl[i] = 0x0;
	}
}

static inline void createMMUTable() {
	/* Flash Memory:
		Begin: 0x0, Length: 32 MiB, Domain: 0, Not Cacheable,
		Not Bufferable */
	mapTable(mmuTable, 0, 32, 0, FALSE, FALSE);
	/* rest of memory bank 1 (unconnected) */
	voidTable(mmuTable, 32, 32);
	/* first MiB of memory bank 2 is unconnected, but set cacheable
	 * used by dCacheCleanAndInvalidate to clean the data data cache */
	mapTable(mmuTable, 64, 1, 0, TRUE, FALSE);
	/* rest and further banks (unconnected) */
	voidTable(mmuTable, 65, 447);
	/* PC Card/Compact Flash Slot 0 and Slot 1:
		Begin: 512MiB, Length: 2 * 256 MiB, Domain: 0,
		Cacheable, Not Bufferable */
	mapTable(mmuTable, 512, 2 * 256, 0, TRUE, FALSE);
	/* Peripherals MMRegs, LCD MMRegs, MemC MMRegs, USB Host MMRegs,
		Capture Interface MMRegs, <reserved>, Internal Memory Control
		Begin: 1024 MiB, Length: 448 MiB, Domain: 0, Not Cacheable,
		Not Bufferable */
	mapTable(mmuTable, 1024, 448, 0, FALSE, FALSE);
	/* SRAM:
		Begin: 1472 MiB, Length: 256 KiB (maximized to 1 MiB), Domain: 0,
		Cacheable, Bufferable */
	mapTable(mmuTable, 1472, 1, 0, TRUE, TRUE);
	/* Rest of internal memory (unusable):
		Begin: 1473 MiB, Length: 63 MiB */
	voidTable(mmuTable, 1473, 63);
	/* Reserved:
		Begin: 1536 MiB, Length: 2 * 512 MiB */
	voidTable(mmuTable, 1536, 2 * 512);
	/* SDRAM:
		Begin: 2560 MiB, Length: 32 MiB, Domain: 0, Cacheable, Bufferable */
	mapTable(mmuTable, 2560, 32, 0, TRUE, TRUE);
	/* Rest is reserved / unconnected */
	voidTable(mmuTable, 2592, 1504);
}

void mmuTableLoad() {
	createMMUTable();
	mmuSetDomain(0x3);
	mmuEnable(mmuTable);
}

