#ifndef SENF_PLATFORM_OS_ARM_BARE_INTERRUPT_H
#define SENF_PLATFORM_OS_ARM_BARE_INTERRUPT_H

#include "inttypes.h"

typedef void (*SEnF_Interrupt_Callback)(uint32_t);

void SEnF_Interrupt_register(uint32_t interrupt, SEnF_Interrupt_Callback callback);
void SEnF_Interrupt_unregister(uint32_t interrupt);

#define MAXINTERRUPT 39

#define SENF_DISABLE_INTERRUPTS() __malloc_lock()
#define SENF_ENABLE_INTERRUPTS() __malloc_unlock()

//compatibility for avr
#define SENF_RESTORE_INTERRUPTS() SENF_ENABLE_INTERRUPTS()

EXTERN_C void __malloc_lock(void);
EXTERN_C void __malloc_unlock(void);

#ifdef DEBUG_INTERRUPT
	#define PRINT_INTERRUPT(_intr) STTHR=(_intr);
	#define PRINT_GPIO() \
		if(GEDR0>0){\
			STTHR=0x10;\
			STTHR=(GEDR0&0xFF000000)>>24;\
			STTHR=(GEDR0&0xFF0000)>>16;\
			STTHR=(GEDR0&0xFF00)>>8;\
			STTHR=GEDR0&0xFF;\
		}\
		if(GEDR1>0){\
			STTHR=0x11;\
			STTHR=(GEDR1&0xFF000000)>>24;\
			STTHR=(GEDR1&0xFF0000)>>16;\
			STTHR=(GEDR1&0xFF00)>>8;\
			STTHR=GEDR1&0xFF;\
		}\
		if(GEDR2>0){\
			STTHR=0x12;\
			STTHR=(GEDR2&0xFF000000)>>24;\
			STTHR=(GEDR2&0xFF0000)>>16;\
			STTHR=(GEDR2&0xFF00)>>8;\
			STTHR=GEDR2&0xFF;\
		}\
		if(GEDR3>0){\
			STTHR=0x13;\
			STTHR=(GEDR3&0xFF000000)>>24;\
			STTHR=(GEDR3&0xFF0000)>>16;\
			STTHR=(GEDR3&0xFF00)>>8;\
			STTHR=GEDR3&0xFF;\
		}
	#ifdef IMOTE_CPU
		#define INTERRUPT_START()\
			STTHR=0xBB;\
			IMOTE_GREEN_ON
	#else
		#define INTERRUPT_START()\
			STTHR=0xBB;
	#endif

	#ifdef IMOTE_CPU
		#define INTERRUPT_END()\
			STTHR=0xDD;\
			IMOTE_GREEN_OFF
	#else
		#define INTERRUPT_END()\
			STTHR=0xDD;
	#endif

#else

	#define PRINT_GPIO()
	#define PRINT_INTERRUPT(_intr)
	#ifdef IMOTE_CPU
		#define INTERRUPT_START() IMOTE_GREEN_ON
	#else
		#define INTERRUPT_START()
	#endif

	#ifdef IMOTE_CPU
		#define INTERRUPT_END() IMOTE_GREEN_OFF
	#else
		#define INTERRUPT_END()
	#endif
#endif

#endif //SENF_PLATFORM_OS_ARM_BARE_INTERRUPT_H
