/* USB Driver: Control Pipe Transfer and State Machine
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Bootloader

The driver contains ideas taken from the linux gadget device driver,
licenced under the GPL.

The Bootloader is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

The Bootloader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef USB_CONTROL_PIPE_INTERNAL_H
#define USB_CONTROL_PIPE_INTERNAL_H

#include <inttypes.h>
#include "usb_configuration.h"

/** \cond INTERNAL */

/** \addtogroup usbcontrol
 * @{
 */

/** \file usb_control_pipe_internal.h
 * \brief USB Control Pipe Internal Structures
 */

extern struct UsbEndpoint usbEndpoint0;

/** \brief control pipe (ep0) states */
enum USB_CONTROL_PIPE_STATE {
	USB_CP_WAIT_FOR_SETUP,
	USB_CP_SETUP_STAGE,
	USB_CP_IN_DATA_STAGE,
	USB_CP_OUT_DATA_STAGE,
	USB_CP_IN_STATUS_STAGE,
	USB_CP_OUT_STATUS_STAGE,
	USB_CP_STALL
};

void usbControlPipeInit();

void usbControlPipeSetState(enum USB_CONTROL_PIPE_STATE cpState);

void usbControlPipeInterrupt(bool pcInt, bool feInt);

/** \brief tells wether a given endpoint is endpoint 0
 * \brief ep endpoint under test
 * \return true iff given endpoint is endpoint 0 (the control pipe) */
static inline bool usbIsControlPipe(const struct UsbEndpoint * ep) {
	return ep == &usbEndpoint0;
}

bool usbControlPipeWrite(struct UsbRequest * req);
bool usbControlPipeRead(struct UsbRequest * req);

void usbControlInRequestComplete(struct UsbRequest * req);
void usbControlOutRequestComplete(struct UsbRequest * req);


/** @} */

/** \endcond */

#endif

